@@ -204,20 +204,21 @@ static bool tracker_is_counter(uvm_tools_event_tracker_t *event_tracker)
     return event_tracker != NULL && !event_tracker->is_queue;
 }
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 2, 0)
-static void put_user_pages(struct page **pages, NvU64 page_count)
+static void uvm_put_user_pages_dirty(struct page **pages, NvU64 page_count)
 {
     NvU64 i;
-    for (i = 0; i < page_count; i++)
+
+    for (i = 0; i < page_count; i++) {
+        set_page_dirty(pages[i]);
         put_page(pages[i]);
+    }
 }
-#endif
 
 static void unmap_user_pages(struct page **pages, void *addr, NvU64 size)
 {
     size = DIV_ROUND_UP(size, PAGE_SIZE);
     vunmap((NvU8 *)addr);
-    put_user_pages(pages, size);
+    uvm_put_user_pages_dirty(pages, size);
     uvm_kvfree(pages);
 }
 
@@ -281,7 +282,7 @@ fail:
     uvm_kvfree(vmas);
 
     if (ret > 0)
-        put_user_pages(*pages, ret);
+        uvm_put_user_pages_dirty(*pages, ret);
     else if (ret < 0)
         status = errno_to_nv_status(ret);
 
@@ -1051,13 +1052,15 @@ void uvm_tools_broadcast_access_counter(uvm_gpu_t *gpu, const uvm_access_counter
     info->eventType           = UvmEventTypeTestAccessCounter;
     info->srcIndex            = gpu->id;
     info->address             = buffer_entry->address.address;
-    info->aperture            = g_hal_to_tools_aperture_table[buffer_entry->address.aperture];
     info->isVirtual           = buffer_entry->address.is_virtual? 1: 0;
     if (buffer_entry->address.is_virtual) {
         info->instancePtr         = buffer_entry->virtual_info.instance_ptr.address;
         info->instancePtrAperture = g_hal_to_tools_aperture_table[buffer_entry->virtual_info.instance_ptr.aperture];
         info->veId                = buffer_entry->virtual_info.ve_id;
     }
+    else {
+        info->aperture            = g_hal_to_tools_aperture_table[buffer_entry->address.aperture];
+    }
     info->isFromCpu           = buffer_entry->counter_type == UVM_ACCESS_COUNTER_TYPE_MOMC? 1: 0;
     info->value               = buffer_entry->counter_value;
     info->subGranularity      = buffer_entry->sub_granularity;
